@@ -25,17 +25,11 @@ COMPONENT_EXPORT(OZONE_SWITCHES) extern const char kUseWaylandExplicitGrab[];
 
 COMPONENT_EXPORT(OZONE_SWITCHES) extern const char kDisableExplicitDmaFences[];
 
-COMPONENT_EXPORT(OZONE_SWITCHES)
-extern const char kDisableRunningAsSystemCompositor[];
-
 COMPONENT_EXPORT(OZONE_SWITCHES)
 extern const char kDisableBufferBWCompression[];
 
 COMPONENT_EXPORT(OZONE_SWITCHES) extern const char kOzoneOverrideScreenSize[];
 
-COMPONENT_EXPORT(OZONE_SWITCHES)
-extern const char kPlatformDisallowsChromeOSDirectVideoDecoder[];
-
 }  // namespace switches
 
 #endif  // UI_OZONE_PUBLIC_OZONE_SWITCHES_H_
