@@ -38,8 +38,7 @@
 #include "ui/views/widget/unique_widget_ptr.h"
 #include "ui/views/widget/widget.h"
 
-namespace views {
-namespace test {
+namespace views::test {
 
 namespace {
 
@@ -201,9 +200,9 @@ class ViewAXPlatformNodeDelegateTest : public ViewsTestBase {
   const int DEFAULT_VIEW_ID = 0;
   const int NON_DEFAULT_VIEW_ID = 1;
 
-  raw_ptr<Widget> widget_ = nullptr;
-  raw_ptr<Button> button_ = nullptr;
-  raw_ptr<Label> label_ = nullptr;
+  raw_ptr<Widget, AcrossTasksDanglingUntriaged> widget_ = nullptr;
+  raw_ptr<Button, AcrossTasksDanglingUntriaged> button_ = nullptr;
+  raw_ptr<Label, AcrossTasksDanglingUntriaged> label_ = nullptr;
   ScopedAXModeSetter ax_mode_setter_;
 };
 
@@ -241,7 +240,8 @@ class ViewAXPlatformNodeDelegateTableTest
 
  private:
   std::unique_ptr<TestTableModel> model_;
-  raw_ptr<TableView> table_ = nullptr;  // Owned by parent.
+  raw_ptr<TableView, AcrossTasksDanglingUntriaged> table_ =
+      nullptr;  // Owned by parent.
 };
 
 class ViewAXPlatformNodeDelegateMenuTest
@@ -298,9 +298,10 @@ class ViewAXPlatformNodeDelegateMenuTest
 
  private:
   // Owned by runner_.
-  raw_ptr<views::TestMenuItemView> menu_ = nullptr;
+  raw_ptr<views::TestMenuItemView, AcrossTasksDanglingUntriaged> menu_ =
+      nullptr;
 
-  raw_ptr<SubmenuView> submenu_ = nullptr;
+  raw_ptr<SubmenuView, AcrossTasksDanglingUntriaged> submenu_ = nullptr;
   std::unique_ptr<TestMenuDelegate> menu_delegate_;
   std::unique_ptr<MenuRunner> runner_;
   UniqueWidgetPtr owner_;
@@ -409,11 +410,8 @@ TEST_F(ViewAXPlatformNodeDelegateTest, GetAuthorUniqueIdNonDefault) {
 
 TEST_F(ViewAXPlatformNodeDelegateTest, OverrideNameAndDescription) {
   // Initially the button has no name and no description.
-  // TODO(accessibility): If nothing has set the name, should the NameFrom
-  // be kNone or kUninitialized instead of kAttribute?
   EXPECT_EQ(button_accessibility()->GetName(), "");
-  EXPECT_EQ(button_accessibility()->GetNameFrom(),
-            ax::mojom::NameFrom::kAttribute);
+  EXPECT_EQ(button_accessibility()->GetNameFrom(), ax::mojom::NameFrom::kNone);
   EXPECT_EQ(button_accessibility()->GetDescription(), "");
   EXPECT_EQ(button_accessibility()->GetDescriptionFrom(),
             ax::mojom::DescriptionFrom::kNone);
@@ -498,13 +496,6 @@ TEST_F(ViewAXPlatformNodeDelegateTest, OverrideNameAndDescription) {
   EXPECT_EQ(button_accessibility()->GetNameFrom(),
             ax::mojom::NameFrom::kRelatedElement);
 
-  // Set the label's View as the description source of the accessible button.
-  // This should also remove the previously-set description.
-  button_accessibility()->OverrideDescribedBy(label_);
-  EXPECT_EQ(button_accessibility()->GetDescription(), "Label's Name");
-  EXPECT_EQ(button_accessibility()->GetDescriptionFrom(),
-            ax::mojom::DescriptionFrom::kRelatedElement);
-
   // Setting the labelledby View to itself should trigger a DCHECK.
   EXPECT_DCHECK_DEATH_WITH(button_accessibility()->OverrideLabelledBy(button_),
                            "Check failed: labelled_by_view != view_");
@@ -932,7 +923,7 @@ TEST_F(ViewAXPlatformNodeDelegateTest, FocusOnMenuClose) {
 }
 
 TEST_F(ViewAXPlatformNodeDelegateTableTest, TableHasHeader) {
-  EXPECT_TRUE(table_accessibility()->GetTableHasColumnOrRowHeaderNode());
+  EXPECT_TRUE(table_accessibility()->TableHasColumnOrRowHeaderNodeForTesting());
   EXPECT_EQ(size_t{4}, table_accessibility()->GetColHeaderNodeIds().size());
   EXPECT_TRUE(table_accessibility()->GetColHeaderNodeIds(5).empty());
 }
@@ -1138,5 +1129,4 @@ TEST_F(AXViewTest, LayoutCalledInvalidateRootView) {
 }
 #endif
 
-}  // namespace test
-}  // namespace views
+}  // namespace views::test
