@@ -6,15 +6,16 @@
 
 #include "ui/views/animation/ink_drop_impl.h"
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/test/gtest_util.h"
 #include "base/test/test_simple_task_runner.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/compositor/layer.h"
 #include "ui/compositor/scoped_animation_duration_scale_mode.h"
 #include "ui/gfx/animation/animation.h"
 #include "ui/gfx/animation/animation_test_api.h"
+#include "ui/native_theme/test_native_theme.h"
 #include "ui/views/animation/ink_drop.h"
 #include "ui/views/animation/ink_drop_ripple.h"
 #include "ui/views/animation/test/ink_drop_impl_test_api.h"
@@ -69,8 +70,10 @@ class InkDropImplTest : public testing::Test {
       base::MakeRefCounted<base::TestSimpleTaskRunner>();
 
   // Required by base::Timer's.
-  std::unique_ptr<base::ThreadTaskRunnerHandle> thread_task_runner_handle_ =
-      std::make_unique<base::ThreadTaskRunnerHandle>(task_runner_);
+  std::unique_ptr<base::SingleThreadTaskRunner::CurrentDefaultHandle>
+      thread_task_runner_handle_ =
+          std::make_unique<base::SingleThreadTaskRunner::CurrentDefaultHandle>(
+              task_runner_);
 
  private:
   TestInkDropHost ink_drop_host_;
@@ -311,6 +314,25 @@ TEST_F(InkDropImplTest, RippleAndHighlightRecreatedOnSizeChange) {
   EXPECT_EQ(bounds.size(), ink_drop_highlight()->layer()->size());
 }
 
+// Make sure the InkDropRipple and InkDropHighlight get recreated when the host
+// theme changes.
+TEST_F(InkDropImplTest, RippleAndHighlightRecreatedOnHostThemeChange) {
+  test_api()->SetShouldHighlight(true);
+  ink_drop()->AnimateToState(InkDropState::ACTIVATED);
+  EXPECT_EQ(1, ink_drop_host()->num_ink_drop_ripples_created());
+  EXPECT_EQ(1, ink_drop_host()->num_ink_drop_highlights_created());
+  EXPECT_EQ(ink_drop_host()->last_ink_drop_ripple(), ink_drop_ripple());
+  EXPECT_EQ(ink_drop_host()->last_ink_drop_highlight(), ink_drop_highlight());
+
+  ui::TestNativeTheme native_theme;
+  native_theme.SetDarkMode(true);
+  ink_drop_host()->SetNativeThemeForTesting(&native_theme);
+  EXPECT_EQ(2, ink_drop_host()->num_ink_drop_ripples_created());
+  EXPECT_EQ(2, ink_drop_host()->num_ink_drop_highlights_created());
+  EXPECT_EQ(ink_drop_host()->last_ink_drop_ripple(), ink_drop_ripple());
+  EXPECT_EQ(ink_drop_host()->last_ink_drop_highlight(), ink_drop_highlight());
+}
+
 // Verifies that the host's GetHighlighted() method reflects the ink drop's
 // highlight state, and when the state changes the ink drop notifies the host.
 TEST_F(InkDropImplTest, HostTracksHighlightState) {
