@@ -13,12 +13,14 @@
 #include "base/i18n/rtl.h"
 #include "base/ranges/algorithm.h"
 #include "base/strings/string_util.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
+#include "third_party/abseil-cpp/absl/types/variant.h"
 #include "ui/accessibility/ax_enums.mojom.h"
-#include "ui/accessibility/ax_node_data.h"
 #include "ui/base/metadata/metadata_impl_macros.h"
 #include "ui/color/color_id.h"
 #include "ui/color/color_provider.h"
 #include "ui/gfx/font_list.h"
+#include "ui/gfx/text_constants.h"
 #include "ui/gfx/text_elider.h"
 #include "ui/gfx/text_utils.h"
 #include "ui/views/controls/label.h"
@@ -76,7 +78,11 @@ struct StyledLabel::LayoutViews {
   std::vector<std::unique_ptr<View>> owned_views;
 };
 
-StyledLabel::StyledLabel() = default;
+StyledLabel::StyledLabel() {
+  SetAccessibilityProperties(text_context_ == style::CONTEXT_DIALOG_TITLE
+                                 ? ax::mojom::Role::kTitleBar
+                                 : ax::mojom::Role::kStaticText);
+}
 
 StyledLabel::~StyledLabel() = default;
 
@@ -93,6 +99,7 @@ void StyledLabel::SetText(std::u16string text) {
     return;
 
   text_ = text;
+  SetAccessibleName(text_);
   style_ranges_.clear();
   RemoveOrDeleteAllChildViews();
   OnPropertyChanged(&text_, kPropertyEffectsPreferredSizeChanged);
@@ -111,7 +118,7 @@ void StyledLabel::AddStyleRange(const gfx::Range& range,
 
   // Insert the new range in sorted order.
   StyleRanges new_range;
-  new_range.push_front(StyleRange(range, style_info));
+  new_range.emplace_front(range, style_info);
   style_ranges_.merge(new_range);
 
   PreferredSizeChanged();
@@ -132,6 +139,9 @@ void StyledLabel::SetTextContext(int text_context) {
     return;
 
   text_context_ = text_context;
+  SetAccessibleRole(text_context_ == style::CONTEXT_DIALOG_TITLE
+                        ? ax::mojom::Role::kTitleBar
+                        : ax::mojom::Role::kStaticText);
   OnPropertyChanged(&text_context_, kPropertyEffectsPreferredSizeChanged);
 }
 
@@ -160,19 +170,20 @@ void StyledLabel::SetLineHeight(int line_height) {
   OnPropertyChanged(&line_height_, kPropertyEffectsPreferredSizeChanged);
 }
 
-absl::optional<SkColor> StyledLabel::GetDisplayedOnBackgroundColor() const {
+StyledLabel::ColorVariant StyledLabel::GetDisplayedOnBackgroundColor() const {
   return displayed_on_background_color_;
 }
 
-void StyledLabel::SetDisplayedOnBackgroundColor(
-    const absl::optional<SkColor>& color) {
-  if (displayed_on_background_color_ == color)
+void StyledLabel::SetDisplayedOnBackgroundColor(ColorVariant color) {
+  if (color == displayed_on_background_color_) {
     return;
+  }
 
   displayed_on_background_color_ = color;
 
-  if (GetWidget())
+  if (GetWidget()) {
     UpdateLabelBackgroundColor();
+  }
 
   OnPropertyChanged(&displayed_on_background_color_, kPropertyEffectsPaint);
 }
@@ -215,13 +226,6 @@ void StyledLabel::SizeToFit(int fixed_width) {
   SetSize(size);
 }
 
-void StyledLabel::GetAccessibleNodeData(ui::AXNodeData* node_data) {
-  node_data->role = (text_context_ == style::CONTEXT_DIALOG_TITLE)
-                        ? ax::mojom::Role::kTitleBar
-                        : ax::mojom::Role::kStaticText;
-  node_data->SetName(GetText());
-}
-
 gfx::Size StyledLabel::CalculatePreferredSize() const {
   // Respect any existing size.  If there is none, default to a single line.
   CalculateLayout((width() == 0) ? std::numeric_limits<int>::max() : width());
@@ -304,11 +308,6 @@ void StyledLabel::PreferredSizeChanged() {
   View::PreferredSizeChanged();
 }
 
-void StyledLabel::OnThemeChanged() {
-  View::OnThemeChanged();
-  UpdateLabelBackgroundColor();
-}
-
 // TODO(wutao): support gfx::ALIGN_TO_HEAD alignment.
 void StyledLabel::SetHorizontalAlignment(gfx::HorizontalAlignment alignment) {
   DCHECK_NE(gfx::ALIGN_TO_HEAD, alignment);
@@ -339,10 +338,14 @@ views::Link* StyledLabel::GetFirstLinkForTesting() {
 
 int StyledLabel::StartX(int excess_space) const {
   int x = GetInsets().left();
-  if (horizontal_alignment_ == gfx::ALIGN_LEFT)
+  // If the element should be aligned to the leading side (left in LTR, or right
+  // in RTL), position it at the leading side Insets (left).
+  if (horizontal_alignment_ ==
+      (base::i18n::IsRTL() ? gfx::ALIGN_RIGHT : gfx::ALIGN_LEFT)) {
     return x;
-  return x + ((horizontal_alignment_ == gfx::ALIGN_CENTER) ? (excess_space / 2)
-                                                           : excess_space);
+  }
+  return x + (horizontal_alignment_ == gfx::ALIGN_CENTER ? (excess_space / 2)
+                                                         : excess_space);
 }
 
 void StyledLabel::CalculateLayout(int width) const {
@@ -570,29 +573,44 @@ std::unique_ptr<Label> StyledLabel::CreateLabel(
         style_info.text_style.value_or(default_text_style_));
   }
 
-  if (style_info.override_color)
+  if (style_info.override_color_id) {
+    result->SetEnabledColorId(style_info.override_color_id.value());
+  } else if (style_info.override_color) {
     result->SetEnabledColor(style_info.override_color.value());
+  }
   if (!style_info.tooltip.empty())
     result->SetTooltipText(style_info.tooltip);
   if (!style_info.accessible_name.empty())
     result->SetAccessibleName(style_info.accessible_name);
-  if (displayed_on_background_color_)
-    result->SetBackgroundColor(displayed_on_background_color_.value());
+  if (absl::holds_alternative<SkColor>(displayed_on_background_color_)) {
+    result->SetBackgroundColor(
+        absl::get<SkColor>(displayed_on_background_color_));
+  } else if (absl::holds_alternative<ui::ColorId>(
+                 displayed_on_background_color_)) {
+    result->SetBackgroundColorId(
+        absl::get<ui::ColorId>(displayed_on_background_color_));
+  }
   result->SetAutoColorReadabilityEnabled(auto_color_readability_enabled_);
   result->SetSubpixelRenderingEnabled(subpixel_rendering_enabled_);
   return result;
 }
 
 void StyledLabel::UpdateLabelBackgroundColor() {
-  SkColor new_color = displayed_on_background_color_.value_or(
-      GetColorProvider()->GetColor(ui::kColorDialogBackground));
   for (View* child : children()) {
     if (!child->GetProperty(kStyledLabelCustomViewKey)) {
       // TODO(kylixrd): Should updating the label background color even be
       // allowed if there are custom views?
       DCHECK((child->GetClassName() == Label::kViewClassName) ||
              (child->GetClassName() == LinkFragment::kViewClassName));
-      static_cast<Label*>(child)->SetBackgroundColor(new_color);
+      static_cast<Label*>(child)->SetBackgroundColorId(
+          absl::holds_alternative<ui::ColorId>(displayed_on_background_color_)
+              ? absl::optional<ui::ColorId>(
+                    absl::get<ui::ColorId>(displayed_on_background_color_))
+              : absl::nullopt);
+      if (absl::holds_alternative<SkColor>(displayed_on_background_color_)) {
+        static_cast<Label*>(child)->SetBackgroundColor(
+            absl::get<SkColor>(displayed_on_background_color_));
+      }
     }
   }
 }
@@ -611,7 +629,7 @@ ADD_PROPERTY_METADATA(int, TextContext)
 ADD_PROPERTY_METADATA(int, DefaultTextStyle)
 ADD_PROPERTY_METADATA(int, LineHeight)
 ADD_PROPERTY_METADATA(bool, AutoColorReadabilityEnabled)
-ADD_PROPERTY_METADATA(absl::optional<SkColor>, DisplayedOnBackgroundColor)
+ADD_PROPERTY_METADATA(StyledLabel::ColorVariant, DisplayedOnBackgroundColor)
 END_METADATA
 
 }  // namespace views
