@@ -180,14 +180,12 @@ class VIEWS_EXPORT TreeView : public View,
   bool HandleAccessibleAction(const ui::AXActionData& action_data) override;
 
   // TreeModelObserver overrides:
-  void TreeNodesAdded(ui::TreeModel* model,
-                      ui::TreeModelNode* parent,
-                      size_t start,
-                      size_t count) override;
-  void TreeNodesRemoved(ui::TreeModel* model,
-                        ui::TreeModelNode* parent,
-                        size_t start,
-                        size_t count) override;
+  void TreeNodeAdded(ui::TreeModel* model,
+                     ui::TreeModelNode* parent,
+                     size_t index) override;
+  void TreeNodeRemoved(ui::TreeModel* model,
+                       ui::TreeModelNode* parent,
+                       size_t index) override;
   void TreeNodeChanged(ui::TreeModel* model,
                        ui::TreeModelNode* model_node) override;
 
@@ -286,14 +284,14 @@ class VIEWS_EXPORT TreeView : public View,
 
    private:
     // The node from the model.
-    raw_ptr<ui::TreeModelNode> model_node_ = nullptr;
+    raw_ptr<ui::TreeModelNode, DanglingUntriaged> model_node_ = nullptr;
 
     // A virtual accessibility view that is used to expose information about
     // this node to assistive software.
     //
     // This is a weak pointer. This class doesn't own its virtual accessibility
     // view but the Views system does.
-    raw_ptr<AXVirtualView> accessibility_view_ = nullptr;
+    raw_ptr<AXVirtualView, DanglingUntriaged> accessibility_view_ = nullptr;
 
     // Whether the children have been loaded.
     bool loaded_children_ = false;
@@ -477,10 +475,10 @@ class VIEWS_EXPORT TreeView : public View,
   InternalNode root_;
 
   // The selected node, may be null.
-  raw_ptr<InternalNode> selected_node_ = nullptr;
+  raw_ptr<InternalNode, DanglingUntriaged> selected_node_ = nullptr;
 
   // The current active node, may be null.
-  raw_ptr<InternalNode> active_node_ = nullptr;
+  raw_ptr<InternalNode, DanglingUntriaged> active_node_ = nullptr;
 
   bool editing_ = false;
 
@@ -503,7 +501,7 @@ class VIEWS_EXPORT TreeView : public View,
   bool editable_ = true;
 
   // The controller.
-  raw_ptr<TreeViewController> controller_ = nullptr;
+  raw_ptr<TreeViewController, DanglingUntriaged> controller_ = nullptr;
 
   // Whether or not the root is shown in the tree.
   bool root_shown_ = true;
