@@ -9,8 +9,8 @@
 #include <memory>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/command_line.h"
+#include "base/functional/bind.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/raw_ptr.h"
 #include "content/public/browser/web_contents.h"
@@ -43,11 +43,7 @@ namespace {
 class WebViewTestWebContentsObserver : public content::WebContentsObserver {
  public:
   explicit WebViewTestWebContentsObserver(content::WebContents* web_contents)
-      : web_contents_(web_contents),
-        was_shown_(false),
-        shown_count_(0),
-        hidden_count_(0),
-        valid_root_while_shown_(true) {
+      : web_contents_(web_contents) {
     content::WebContentsObserver::Observe(web_contents);
   }
 
@@ -100,11 +96,11 @@ class WebViewTestWebContentsObserver : public content::WebContentsObserver {
 
  private:
   raw_ptr<content::WebContents> web_contents_;
-  bool was_shown_;
-  int32_t shown_count_;
-  int32_t hidden_count_;
+  bool was_shown_ = false;
+  int32_t shown_count_ = 0;
+  int32_t hidden_count_ = 0;
   // Set to true if the view containing the webcontents has a valid root window.
-  bool valid_root_while_shown_;
+  bool valid_root_while_shown_ = true;
 };
 
 // Fakes the fullscreen browser state reported to WebContents and WebView.
@@ -131,6 +127,20 @@ class WebViewTestWebContentsDelegate : public content::WebContentsDelegate {
   bool is_fullscreened_ = false;
 };
 
+void SimulateRendererCrash(content::WebContents* contents, WebView* view) {
+  auto* tester = content::WebContentsTester::For(contents);
+
+  // Normally when a renderer crashes, the WebView will learn about it
+  // automatically via WebContentsObserver. Since this is a test
+  // WebContents, simulate that by calling SetIsCrashed and then
+  // explicitly calling RenderFrameDeleted on the WebView to trigger it
+  // to swap in the crashed overlay view.
+  tester->SetIsCrashed(base::TERMINATION_STATUS_PROCESS_CRASHED, -1);
+  EXPECT_TRUE(contents->IsCrashed());
+  static_cast<content::WebContentsObserver*>(view)->RenderFrameDeleted(
+      contents->GetPrimaryMainFrame());
+}
+
 }  // namespace
 
 // Provides functionality to test a WebView.
@@ -205,7 +215,7 @@ class WebViewUnitTest : public views::test::WidgetTest {
 
   std::unique_ptr<content::WebContents> CreateTestWebContents() const {
     return content::WebContentsTester::CreateTestWebContents(
-        browser_context_.get(), /*site_instnace=*/nullptr);
+        browser_context_.get(), /*instance=*/nullptr);
   }
 
   void SetAXMode(ui::AXMode mode) { web_view()->OnAXModeAdded(mode); }
@@ -217,8 +227,8 @@ class WebViewUnitTest : public views::test::WidgetTest {
   std::unique_ptr<views::WebView::ScopedWebContentsCreatorForTesting>
       scoped_web_contents_creator_;
 
-  raw_ptr<Widget> top_level_widget_ = nullptr;
-  raw_ptr<WebView> web_view_ = nullptr;
+  raw_ptr<Widget, DanglingUntriaged> top_level_widget_ = nullptr;
+  raw_ptr<WebView, DanglingUntriaged> web_view_ = nullptr;
 };
 
 // Tests that attaching and detaching a WebContents to a WebView makes the
@@ -322,18 +332,14 @@ TEST_F(WebViewUnitTest, DetachedWebViewDestructor) {
   // Init WebView with attached NativeView.
   const std::unique_ptr<content::WebContents> web_contents =
       CreateWebContents();
-  std::unique_ptr<WebView> webview(
-      new WebView(web_contents->GetBrowserContext()));
   View* contents_view = top_level_widget()->GetContentsView();
-  contents_view->AddChildView(webview.get());
-  webview->SetWebContents(web_contents.get());
+  auto* web_view = contents_view->AddChildView(
+      std::make_unique<WebView>(web_contents->GetBrowserContext()));
 
   // Remove WebView from views hierarchy. NativeView should be detached
-  // from Widget.
-  contents_view->RemoveChildView(webview.get());
-  // Destroy WebView. NativeView should be detached secondary.
-  // There should be no crash.
-  webview.reset();
+  // from Widget, and the WebView should be subsequently destroyed with no
+  // crash.
+  contents_view->RemoveChildViewT(web_view);
 }
 
 // Test that the specified crashed overlay view is shown when a WebContents
@@ -341,60 +347,21 @@ TEST_F(WebViewUnitTest, DetachedWebViewDestructor) {
 TEST_F(WebViewUnitTest, CrashedOverlayView) {
   const std::unique_ptr<content::WebContents> web_contents =
       CreateTestWebContents();
-  content::WebContentsTester* tester =
-      content::WebContentsTester::For(web_contents.get());
 
-  std::unique_ptr<WebView> web_view(
-      new WebView(web_contents->GetBrowserContext()));
   View* contents_view = top_level_widget()->GetContentsView();
-  contents_view->AddChildView(web_view.get());
+  auto* web_view = contents_view->AddChildView(
+      std::make_unique<WebView>(web_contents->GetBrowserContext()));
   web_view->SetWebContents(web_contents.get());
 
-  View* crashed_overlay_view = new View();
-  web_view->SetCrashedOverlayView(crashed_overlay_view);
-  EXPECT_FALSE(crashed_overlay_view->IsDrawn());
-
-  // Normally when a renderer crashes, the WebView will learn about it
-  // automatically via WebContentsObserver. Since this is a test
-  // WebContents, simulate that by calling SetIsCrashed and then
-  // explicitly calling RenderFrameDeleted on the WebView to trigger it
-  // to swap in the crashed overlay view.
-  tester->SetIsCrashed(base::TERMINATION_STATUS_PROCESS_CRASHED, -1);
-  EXPECT_TRUE(web_contents->IsCrashed());
-  static_cast<content::WebContentsObserver*>(web_view.get())
-      ->RenderFrameDeleted(web_contents->GetPrimaryMainFrame());
-  EXPECT_TRUE(crashed_overlay_view->IsDrawn());
-}
-
-// Test that a crashed overlay view isn't deleted if it's owned by client.
-TEST_F(WebViewUnitTest, CrashedOverlayViewOwnedbyClient) {
-  const std::unique_ptr<content::WebContents> web_contents =
-      CreateTestWebContents();
-  content::WebContentsTester* tester =
-      content::WebContentsTester::For(web_contents.get());
-  std::unique_ptr<WebView> web_view(
-      new WebView(web_contents->GetBrowserContext()));
-  View* contents_view = top_level_widget()->GetContentsView();
-  contents_view->AddChildView(web_view.get());
-  web_view->SetWebContents(web_contents.get());
-
-  View* crashed_overlay_view = new View();
+  auto crashed_overlay_view = std::make_unique<View>();
   crashed_overlay_view->set_owned_by_client();
-  web_view->SetCrashedOverlayView(crashed_overlay_view);
+  web_view->SetCrashedOverlayView(crashed_overlay_view.get());
   EXPECT_FALSE(crashed_overlay_view->IsDrawn());
 
-  // Simulate a renderer crash (see above).
-  tester->SetIsCrashed(base::TERMINATION_STATUS_PROCESS_CRASHED, -1);
-  EXPECT_TRUE(web_contents->IsCrashed());
-  static_cast<content::WebContentsObserver*>(web_view.get())
-      ->RenderFrameDeleted(web_contents->GetPrimaryMainFrame());
+  SimulateRendererCrash(web_contents.get(), web_view);
   EXPECT_TRUE(crashed_overlay_view->IsDrawn());
 
   web_view->SetCrashedOverlayView(nullptr);
-  web_view.reset();
-
-  // This shouldn't crash, we still own this.
-  delete crashed_overlay_view;
 }
 
 // Tests to make sure we can default construct the WebView class and set the
@@ -449,13 +416,20 @@ TEST_F(WebViewUnitTest, ReparentingUpdatesParentAccessible) {
 
 // This tests that we don't crash if WebView doesn't have a Widget or a
 // Webcontents. https://crbug.com/1191999
-TEST_F(WebViewUnitTest, ChangeAXMode) {
+// TODO(crbug.com/1465744): Re-enable this test
+#if BUILDFLAG(IS_LINUX)
+#define MAYBE_ChangeAXMode DISABLED_ChangeAXMode
+#else
+#define MAYBE_ChangeAXMode ChangeAXMode
+#endif
+TEST_F(WebViewUnitTest, MAYBE_ChangeAXMode) {
   // Case 1: WebView has a Widget and no WebContents.
   SetAXMode(ui::AXMode::kFirstModeFlag);
 
   // Case 2: WebView has no Widget and a WebContents.
   View* contents_view = top_level_widget()->GetContentsView();
-  contents_view->RemoveChildView(web_view());
+  // Remove the view but make sure to delete it at the end of the test.
+  auto scoped_view = contents_view->RemoveChildViewT(web_view());
   const std::unique_ptr<content::WebContents> web_contents =
       CreateWebContents();
   web_view()->SetWebContents(web_contents.get());
