@@ -8,8 +8,7 @@
 #include "base/no_destructor.h"
 #include "base/observer_list.h"
 
-namespace views {
-namespace internal {
+namespace views::internal {
 
 // static
 AnyWidgetObserverSingleton* AnyWidgetObserverSingleton::GetInstance() {
@@ -41,5 +40,4 @@ void AnyWidgetObserverSingleton::RemoveObserver(AnyWidgetObserver* observer) {
 AnyWidgetObserverSingleton::AnyWidgetObserverSingleton() = default;
 AnyWidgetObserverSingleton::~AnyWidgetObserverSingleton() = default;
 
-}  // namespace internal
-}  // namespace views
+}  // namespace views::internal
