@@ -4,16 +4,18 @@
 
 #include "ui/views/widget/widget.h"
 
+#include <algorithm>
 #include <set>
 #include <utility>
 
 #include "base/auto_reset.h"
-#include "base/bind.h"
 #include "base/check_op.h"
 #include "base/containers/adapters.h"
+#include "base/functional/bind.h"
 #include "base/i18n/rtl.h"
 #include "base/notreached.h"
 #include "base/observer_list.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
@@ -25,6 +27,7 @@
 #include "ui/base/metadata/metadata_impl_macros.h"
 #include "ui/base/models/image_model.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/color/color_provider_manager.h"
 #include "ui/compositor/compositor.h"
 #include "ui/compositor/layer.h"
 #include "ui/display/screen.h"
@@ -32,6 +35,7 @@
 #include "ui/events/event_utils.h"
 #include "ui/gfx/image/image_skia.h"
 #include "ui/views/controls/menu/menu_controller.h"
+#include "ui/views/drag_controller.h"
 #include "ui/views/event_monitor.h"
 #include "ui/views/focus/focus_manager.h"
 #include "ui/views/focus/focus_manager_factory.h"
@@ -190,13 +194,17 @@ Widget::~Widget() {
   if (widget_delegate_)
     widget_delegate_->WidgetDestroying();
   if (ownership_ == InitParams::WIDGET_OWNS_NATIVE_WIDGET) {
-    delete native_widget_;
-    DCHECK(native_widget_destroyed_);
-  } else {
+    owned_native_widget_.reset();
+    DCHECK(!native_widget_);
+  } else if (ownership_ == InitParams::NATIVE_WIDGET_OWNS_WIDGET) {
     // TODO(crbug.com/937381): Revert to DCHECK once we figure out the reason.
-    CHECK(native_widget_destroyed_)
+    CHECK(!native_widget_)
         << "Destroying a widget with a live native widget. "
         << "Widget probably should use WIDGET_OWNS_NATIVE_WIDGET ownership.";
+  } else {
+    DCHECK_EQ(ownership_, InitParams::CLIENT_OWNS_WIDGET);
+    if (native_widget_)
+      native_widget_->Close();
   }
   // Destroy RootView after the native widget, so in case the WidgetDelegate is
   // a View in the RootView hierarchy it gets destroyed as a WidgetDelegate
@@ -248,6 +256,9 @@ Widget* Widget::CreateWindowWithContext(
 
 // static
 Widget* Widget::GetWidgetForNativeView(gfx::NativeView native_view) {
+  if (!native_view)
+    return nullptr;
+
   internal::NativeWidgetPrivate* native_widget =
       internal::NativeWidgetPrivate::GetNativeWidgetForNativeView(native_view);
   return native_widget ? native_widget->GetWidget() : nullptr;
@@ -255,6 +266,9 @@ Widget* Widget::GetWidgetForNativeView(gfx::NativeView native_view) {
 
 // static
 Widget* Widget::GetWidgetForNativeWindow(gfx::NativeWindow native_window) {
+  if (!native_window)
+    return nullptr;
+
   internal::NativeWidgetPrivate* native_widget =
       internal::NativeWidgetPrivate::GetNativeWidgetForNativeWindow(
           native_window);
@@ -263,6 +277,9 @@ Widget* Widget::GetWidgetForNativeWindow(gfx::NativeWindow native_window) {
 
 // static
 Widget* Widget::GetTopLevelWidgetForNativeView(gfx::NativeView native_view) {
+  if (!native_view)
+    return nullptr;
+
   internal::NativeWidgetPrivate* native_widget =
       internal::NativeWidgetPrivate::GetTopLevelNativeWidget(native_view);
   return native_widget ? native_widget->GetWidget() : nullptr;
@@ -271,17 +288,24 @@ Widget* Widget::GetTopLevelWidgetForNativeView(gfx::NativeView native_view) {
 // static
 void Widget::GetAllChildWidgets(gfx::NativeView native_view,
                                 Widgets* children) {
+  if (!native_view)
+    return;
+
   internal::NativeWidgetPrivate::GetAllChildWidgets(native_view, children);
 }
 
 // static
 void Widget::GetAllOwnedWidgets(gfx::NativeView native_view, Widgets* owned) {
+  if (!native_view)
+    return;
+
   internal::NativeWidgetPrivate::GetAllOwnedWidgets(native_view, owned);
 }
 
 // static
 void Widget::ReparentNativeView(gfx::NativeView native_view,
                                 gfx::NativeView new_parent) {
+  DCHECK(native_view);
   internal::NativeWidgetPrivate::ReparentNativeView(native_view, new_parent);
   Widget* child_widget = GetWidgetForNativeView(native_view);
   Widget* parent_widget =
@@ -355,14 +379,24 @@ void Widget::Init(InitParams params) {
     // ViewsDelegate::OnBeforeWidgetInit() may change `params.delegate` either
     // by setting it to null or assigning a different value to it, so handle
     // both cases.
-    auto default_widget_delegate = std::make_unique<DefaultWidgetDelegate>();
-    widget_delegate_ =
-        params.delegate ? params.delegate : default_widget_delegate.get();
+    // TODO(kylixrd): Rework this to avoid always creating the default delegate
+    // once the widget never owns a provided delegate.
+    owned_widget_delegate_ = std::make_unique<DefaultWidgetDelegate>();
+    widget_delegate_ = params.delegate ? params.delegate->AsWeakPtr()
+                                       : owned_widget_delegate_->AsWeakPtr();
 
     ViewsDelegate::GetInstance()->OnBeforeWidgetInit(&params, this);
 
-    widget_delegate_ =
-        params.delegate ? params.delegate : default_widget_delegate.release();
+    widget_delegate_ = params.delegate ? params.delegate->AsWeakPtr()
+                                       : owned_widget_delegate_->AsWeakPtr();
+    if (widget_delegate_.get() != owned_widget_delegate_.get()) {
+      // TODO(kylixrd): This will be unnecessary once the Widget can no longer
+      // "own" the delegate.
+      if (widget_delegate_->owned_by_widget())
+        owned_widget_delegate_ = base::WrapUnique(widget_delegate_.get());
+      else
+        owned_widget_delegate_.reset();
+    }
   }
   DCHECK(widget_delegate_);
 
@@ -378,8 +412,7 @@ void Widget::Init(InitParams params) {
   // Henceforth, ensure the delegate outlives the Widget.
   widget_delegate_->can_delete_this_ = false;
 
-  if (params.delegate)
-    params.delegate->WidgetInitializing(this);
+  widget_delegate_->WidgetInitializing(this);
 
   ownership_ = params.ownership;
 
@@ -392,7 +425,16 @@ void Widget::Init(InitParams params) {
         std::make_unique<SublevelManager>(this, params.sublevel);
   }
 
-  native_widget_ = CreateNativeWidget(params, this)->AsNativeWidgetPrivate();
+  if (params.native_theme) {
+    native_theme_ = params.native_theme;
+  }
+
+  internal::NativeWidgetPrivate* native_widget_raw_ptr =
+      CreateNativeWidget(params, this)->AsNativeWidgetPrivate();
+  native_widget_ = native_widget_raw_ptr->GetWeakPtr();
+  if (params.ownership == InitParams::WIDGET_OWNS_NATIVE_WIDGET) {
+    owned_native_widget_ = base::WrapUnique(native_widget_raw_ptr);
+  }
   root_view_.reset(CreateRootView());
 
   // Copy the elements of params that will be used after it is moved.
@@ -456,21 +498,19 @@ void Widget::Init(InitParams params) {
 }
 
 void Widget::ShowEmojiPanel() {
-  native_widget_->ShowEmojiPanel();
+  if (native_widget_)
+    native_widget_->ShowEmojiPanel();
 }
 
 // Unconverted methods (see header) --------------------------------------------
 
 gfx::NativeView Widget::GetNativeView() const {
-  if (native_widget_destroyed_)
-    return gfx::kNullNativeView;
-  return native_widget_->GetNativeView();
+  return native_widget_ ? native_widget_->GetNativeView() : gfx::NativeView();
 }
 
 gfx::NativeWindow Widget::GetNativeWindow() const {
-  if (native_widget_destroyed_)
-    return gfx::kNullNativeWindow;
-  return native_widget_->GetNativeWindow();
+  return native_widget_ ? native_widget_->GetNativeWindow()
+                        : gfx::NativeWindow();
 }
 
 void Widget::AddObserver(WidgetObserver* observer) {
@@ -510,12 +550,14 @@ void Widget::ViewHierarchyChanged(const ViewHierarchyChangedDetails& details) {
     FocusManager* focus_manager = GetFocusManager();
     if (focus_manager)
       focus_manager->ViewRemoved(details.child);
-    if (!native_widget_destroyed_)
+    if (native_widget_)
       native_widget_->ViewRemoved(details.child);
   }
 }
 
 void Widget::NotifyNativeViewHierarchyWillChange() {
+  if (!native_widget_)
+    return;
   // During tear-down the top-level focus manager becomes unavailable to
   // GTK tabbed panes and their children, so normal deregistration via
   // |FocusManager::ViewRemoved()| calls are fouled.  We clear focus here
@@ -526,6 +568,8 @@ void Widget::NotifyNativeViewHierarchyWillChange() {
 }
 
 void Widget::NotifyNativeViewHierarchyChanged() {
+  if (!native_widget_)
+    return;
   native_widget_->OnNativeViewHierarchyChanged();
   root_view_->NotifyNativeViewHierarchyChanged();
 }
@@ -549,8 +593,7 @@ const Widget* Widget::GetTopLevelWidget() const {
   // focus manager.
   if (is_top_level())
     return this;
-  return native_widget_destroyed_ ? nullptr
-                                  : native_widget_->GetTopLevelWidget();
+  return native_widget_ ? native_widget_->GetTopLevelWidget() : nullptr;
 }
 
 Widget* Widget::GetPrimaryWindowWidget() {
@@ -588,82 +631,99 @@ View* Widget::GetContentsView() {
 }
 
 gfx::Rect Widget::GetWindowBoundsInScreen() const {
-  return native_widget_->GetWindowBoundsInScreen();
+  return native_widget_ ? native_widget_->GetWindowBoundsInScreen()
+                        : gfx::Rect();
 }
 
 gfx::Rect Widget::GetClientAreaBoundsInScreen() const {
-  return native_widget_->GetClientAreaBoundsInScreen();
+  return native_widget_ ? native_widget_->GetClientAreaBoundsInScreen()
+                        : gfx::Rect();
 }
 
 gfx::Rect Widget::GetRestoredBounds() const {
-  return native_widget_->GetRestoredBounds();
+  return native_widget_ ? native_widget_->GetRestoredBounds() : gfx::Rect();
 }
 
 std::string Widget::GetWorkspace() const {
-  return native_widget_->GetWorkspace();
+  return native_widget_ ? native_widget_->GetWorkspace() : "";
 }
 
 void Widget::SetBounds(const gfx::Rect& bounds) {
-  native_widget_->SetBounds(bounds);
+  if (native_widget_)
+    native_widget_->SetBounds(bounds);
 }
 
 void Widget::SetSize(const gfx::Size& size) {
-  native_widget_->SetSize(size);
+  if (native_widget_)
+    native_widget_->SetSize(size);
 }
 
 void Widget::CenterWindow(const gfx::Size& size) {
-  native_widget_->CenterWindow(size);
+  if (native_widget_)
+    native_widget_->CenterWindow(size);
 }
 
 void Widget::SetBoundsConstrained(const gfx::Rect& bounds) {
-  native_widget_->SetBoundsConstrained(bounds);
+  if (native_widget_)
+    native_widget_->SetBoundsConstrained(bounds);
 }
 
 void Widget::SetVisibilityChangedAnimationsEnabled(bool value) {
-  native_widget_->SetVisibilityChangedAnimationsEnabled(value);
+  if (native_widget_)
+    native_widget_->SetVisibilityChangedAnimationsEnabled(value);
 }
 
 void Widget::SetVisibilityAnimationDuration(const base::TimeDelta& duration) {
-  native_widget_->SetVisibilityAnimationDuration(duration);
+  if (native_widget_)
+    native_widget_->SetVisibilityAnimationDuration(duration);
 }
 
 void Widget::SetVisibilityAnimationTransition(VisibilityTransition transition) {
-  native_widget_->SetVisibilityAnimationTransition(transition);
+  if (native_widget_)
+    native_widget_->SetVisibilityAnimationTransition(transition);
 }
 
 bool Widget::IsMoveLoopSupported() const {
-  return native_widget_->IsMoveLoopSupported();
+  return native_widget_ ? native_widget_->IsMoveLoopSupported() : false;
 }
 
 Widget::MoveLoopResult Widget::RunMoveLoop(
     const gfx::Vector2d& drag_offset,
     MoveLoopSource source,
     MoveLoopEscapeBehavior escape_behavior) {
+  if (!native_widget_)
+    return MoveLoopResult::kCanceled;
+
   return native_widget_->RunMoveLoop(drag_offset, source, escape_behavior);
 }
 
 void Widget::EndMoveLoop() {
-  native_widget_->EndMoveLoop();
+  if (native_widget_)
+    native_widget_->EndMoveLoop();
 }
 
 void Widget::StackAboveWidget(Widget* widget) {
-  native_widget_->StackAbove(widget->GetNativeView());
+  if (native_widget_)
+    native_widget_->StackAbove(widget->GetNativeView());
 }
 
 void Widget::StackAbove(gfx::NativeView native_view) {
-  native_widget_->StackAbove(native_view);
+  if (native_widget_)
+    native_widget_->StackAbove(native_view);
 }
 
 void Widget::StackAtTop() {
-  native_widget_->StackAtTop();
+  if (native_widget_)
+    native_widget_->StackAtTop();
 }
 
 bool Widget::IsStackedAbove(gfx::NativeView native_view) {
-  return native_widget_->IsStackedAbove(native_view);
+  return native_widget_ ? native_widget_->IsStackedAbove(native_view) : false;
 }
 
 void Widget::SetShape(std::unique_ptr<ShapeRects> shape) {
-  native_widget_->SetShape(std::move(shape));
+  if (native_widget_)
+    native_widget_->SetShape(std::move(shape));
 }
 
 void Widget::CloseWithReason(ClosedReason closed_reason) {
@@ -709,7 +769,8 @@ void Widget::CloseWithReason(ClosedReason closed_reason) {
   if (widget_delegate_)
     widget_delegate_->WindowWillClose();
 
-  native_widget_->Close();
+  if (native_widget_)
+    native_widget_->Close();
 }
 
 void Widget::Close() {
@@ -725,9 +786,9 @@ void Widget::CloseNow() {
     observer.OnWidgetClosing(this);
   internal::AnyWidgetObserverSingleton::GetInstance()->OnAnyWidgetClosing(this);
 
-  DCHECK(native_widget_) << "Native widget is never initialized.";
+  DCHECK(native_widget_initialized_) << "Native widget is never initialized.";
 
-  if (!native_widget_destroyed_)
+  if (native_widget_)
     native_widget_->CloseNow();
 }
 
@@ -736,6 +797,8 @@ bool Widget::IsClosed() const {
 }
 
 void Widget::Show() {
+  if (!native_widget_)
+    return;
   const ui::Layer* layer = GetLayer();
   TRACE_EVENT1("views", "Widget::Show", "layer",
                layer ? layer->name() : "none");
@@ -765,11 +828,15 @@ void Widget::Show() {
 }
 
 void Widget::Hide() {
+  if (!native_widget_)
+    return;
   native_widget_->Hide();
   internal::AnyWidgetObserverSingleton::GetInstance()->OnAnyWidgetHidden(this);
 }
 
 void Widget::ShowInactive() {
+  if (!native_widget_)
+    return;
   // If this gets called with saved_show_state_ == ui::SHOW_STATE_MAXIMIZED,
   // call SetBounds()with the restored bounds to set the correct size. This
   // normally should not happen, but if it does we should avoid showing unsized
@@ -785,24 +852,27 @@ void Widget::ShowInactive() {
 }
 
 void Widget::Activate() {
-  if (CanActivate())
+  if (CanActivate() && native_widget_)
     native_widget_->Activate();
 }
 
 void Widget::Deactivate() {
-  native_widget_->Deactivate();
+  if (native_widget_)
+    native_widget_->Deactivate();
 }
 
 bool Widget::IsActive() const {
-  return native_widget_->IsActive();
+  return native_widget_ ? native_widget_->IsActive() : false;
 }
 
 void Widget::SetZOrderLevel(ui::ZOrderLevel order) {
-  native_widget_->SetZOrderLevel(order);
+  if (native_widget_)
+    native_widget_->SetZOrderLevel(order);
 }
 
 ui::ZOrderLevel Widget::GetZOrderLevel() const {
-  return native_widget_->GetZOrderLevel();
+  return native_widget_ ? native_widget_->GetZOrderLevel()
+                        : ui::ZOrderLevel::kNormal;
 }
 
 void Widget::SetZOrderSublevel(int sublevel) {
@@ -810,38 +880,47 @@ void Widget::SetZOrderSublevel(int sublevel) {
 }
 
 int Widget::GetZOrderSublevel() const {
+  if (!sublevel_manager_)
+    return 0;
+
   return sublevel_manager_->GetSublevel();
 }
 
 void Widget::SetVisibleOnAllWorkspaces(bool always_visible) {
-  native_widget_->SetVisibleOnAllWorkspaces(always_visible);
+  if (native_widget_)
+    native_widget_->SetVisibleOnAllWorkspaces(always_visible);
 }
 
 bool Widget::IsVisibleOnAllWorkspaces() const {
-  return native_widget_->IsVisibleOnAllWorkspaces();
+  return native_widget_ ? native_widget_->IsVisibleOnAllWorkspaces() : false;
 }
 
 void Widget::Maximize() {
-  native_widget_->Maximize();
+  if (native_widget_)
+    native_widget_->Maximize();
 }
 
 void Widget::Minimize() {
-  native_widget_->Minimize();
+  if (native_widget_)
+    native_widget_->Minimize();
 }
 
 void Widget::Restore() {
-  native_widget_->Restore();
+  if (native_widget_)
+    native_widget_->Restore();
 }
 
 bool Widget::IsMaximized() const {
-  return native_widget_->IsMaximized();
+  return native_widget_ ? native_widget_->IsMaximized() : false;
 }
 
 bool Widget::IsMinimized() const {
-  return native_widget_->IsMinimized();
+  return native_widget_ ? native_widget_->IsMinimized() : false;
 }
 
 void Widget::SetFullscreen(bool fullscreen, int64_t target_display_id) {
+  if (!native_widget_)
+    return;
   // It isn't valid to specify `target_display_id` when exiting fullscreen.
   if (!fullscreen)
     DCHECK(target_display_id == display::kInvalidDisplayId);
@@ -860,27 +939,43 @@ void Widget::SetFullscreen(bool fullscreen, int64_t target_display_id) {
 }
 
 bool Widget::IsFullscreen() const {
-  return native_widget_->IsFullscreen();
+  return native_widget_ ? native_widget_->IsFullscreen() : false;
 }
 
 void Widget::SetCanAppearInExistingFullscreenSpaces(
     bool can_appear_in_existing_fullscreen_spaces) {
-  native_widget_->SetCanAppearInExistingFullscreenSpaces(
-      can_appear_in_existing_fullscreen_spaces);
+  if (native_widget_) {
+    native_widget_->SetCanAppearInExistingFullscreenSpaces(
+        can_appear_in_existing_fullscreen_spaces);
+  }
 }
 
 void Widget::SetOpacity(float opacity) {
   DCHECK(opacity >= 0.0f);
   DCHECK(opacity <= 1.0f);
-  native_widget_->SetOpacity(opacity);
+  if (native_widget_)
+    native_widget_->SetOpacity(opacity);
 }
 
 void Widget::SetAspectRatio(const gfx::SizeF& aspect_ratio) {
-  native_widget_->SetAspectRatio(aspect_ratio);
+  if (!native_widget_) {
+    return;
+  }
+
+  // The aspect ratio affects the client view only, so figure out how much of
+  // the widget isn't taken up by the client view.
+  gfx::Size excluded_margin;
+  if (non_client_view() && non_client_view()->frame_view()) {
+    excluded_margin =
+        non_client_view()->bounds().size() -
+        non_client_view()->frame_view()->GetBoundsForClientView().size();
+  }
+  native_widget_->SetAspectRatio(aspect_ratio, excluded_margin);
 }
 
 void Widget::FlashFrame(bool flash) {
-  native_widget_->FlashFrame(flash);
+  if (native_widget_)
+    native_widget_->FlashFrame(flash);
 }
 
 View* Widget::GetRootView() {
@@ -892,7 +987,7 @@ const View* Widget::GetRootView() const {
 }
 
 bool Widget::IsVisible() const {
-  return native_widget_->IsVisible();
+  return native_widget_ ? native_widget_->IsVisible() : false;
 }
 
 const ui::ThemeProvider* Widget::GetThemeProvider() const {
@@ -905,8 +1000,7 @@ const ui::ThemeProvider* Widget::GetThemeProvider() const {
                                               : nullptr;
 }
 
-ui::ColorProviderManager::ThemeInitializerSupplier* Widget::GetCustomTheme()
-    const {
+ui::ColorProviderKey::ThemeInitializerSupplier* Widget::GetCustomTheme() const {
   return nullptr;
 }
 
@@ -921,7 +1015,7 @@ const FocusManager* Widget::GetFocusManager() const {
 }
 
 ui::InputMethod* Widget::GetInputMethod() {
-  if (is_top_level()) {
+  if (is_top_level() && native_widget_) {
     // Only creates the shared the input method instance on top level widget.
     return native_widget_private()->GetInputMethod();
   } else {
@@ -945,12 +1039,18 @@ void Widget::RunShellDrag(View* view,
                           const gfx::Point& location,
                           int operation,
                           ui::mojom::DragEventSource source) {
+  if (!native_widget_)
+    return;
   dragged_view_ = view;
   OnDragWillStart();
 
   for (WidgetObserver& observer : observers_)
     observer.OnWidgetDragWillStart(this);
 
+  if (view && view->drag_controller()) {
+    view->drag_controller()->OnWillStartDragForView(view);
+  }
+
   WidgetDeletionObserver widget_deletion_observer(this);
   native_widget_->RunShellDrag(view, std::move(data), location, operation,
                                source);
@@ -975,32 +1075,38 @@ void Widget::SchedulePaintInRect(const gfx::Rect& rect) {
   // This happens when DestroyRootView removes all children from the
   // RootView which triggers a SchedulePaint that ends up here. This happens
   // after in ~Widget after native_widget_ is destroyed.
-  if (native_widget_destroyed_)
-    return;
-  native_widget_->SchedulePaintInRect(rect);
+  if (native_widget_)
+    native_widget_->SchedulePaintInRect(rect);
 }
 
 void Widget::ScheduleLayout() {
-  native_widget_->ScheduleLayout();
+  if (native_widget_)
+    native_widget_->ScheduleLayout();
 }
 
 void Widget::SetCursor(const ui::Cursor& cursor) {
-  native_widget_->SetCursor(cursor);
+  if (native_widget_)
+    native_widget_->SetCursor(cursor);
 }
 
 bool Widget::IsMouseEventsEnabled() const {
-  return native_widget_->IsMouseEventsEnabled();
+  return native_widget_ ? native_widget_->IsMouseEventsEnabled() : false;
 }
 
 void Widget::SetNativeWindowProperty(const char* name, void* value) {
-  native_widget_->SetNativeWindowProperty(name, value);
+  if (native_widget_)
+    native_widget_->SetNativeWindowProperty(name, value);
 }
 
 void* Widget::GetNativeWindowProperty(const char* name) const {
-  return native_widget_->GetNativeWindowProperty(name);
+  return native_widget_ ? native_widget_->GetNativeWindowProperty(name)
+                        : nullptr;
 }
 
 void Widget::UpdateWindowTitle() {
+  if (!native_widget_)
+    return;
+
   if (!non_client_view_)
     return;
 
@@ -1015,6 +1121,9 @@ void Widget::UpdateWindowTitle() {
 }
 
 void Widget::UpdateWindowIcon() {
+  if (!native_widget_)
+    return;
+
   if (non_client_view_)
     non_client_view_->UpdateWindowIcon();
 
@@ -1073,10 +1182,13 @@ void Widget::SetFocusTraversableParentView(View* parent_view) {
 }
 
 void Widget::ClearNativeFocus() {
-  native_widget_->ClearNativeFocus();
+  if (native_widget_)
+    native_widget_->ClearNativeFocus();
 }
 
 std::unique_ptr<NonClientFrameView> Widget::CreateNonClientFrameView() {
+  if (!native_widget_)
+    return nullptr;
   auto frame_view = widget_delegate_->CreateNonClientFrameView(this);
   if (!frame_view)
     frame_view = native_widget_->CreateNonClientFrameView();
@@ -1093,45 +1205,30 @@ std::unique_ptr<NonClientFrameView> Widget::CreateNonClientFrameView() {
 bool Widget::ShouldUseNativeFrame() const {
   if (frame_type_ != FrameType::kDefault)
     return frame_type_ == FrameType::kForceNative;
-  return native_widget_->ShouldUseNativeFrame();
+  return native_widget_ ? native_widget_->ShouldUseNativeFrame() : false;
 }
 
 bool Widget::ShouldWindowContentsBeTransparent() const {
-  return native_widget_->ShouldWindowContentsBeTransparent();
-}
-
-void Widget::DebugToggleFrameType() {
-  if (frame_type_ == FrameType::kDefault) {
-    frame_type_ = ShouldUseNativeFrame() ? FrameType::kForceCustom
-                                         : FrameType::kForceNative;
-  } else {
-    frame_type_ = frame_type_ == FrameType::kForceCustom
-                      ? FrameType::kForceNative
-                      : FrameType::kForceCustom;
-  }
-  FrameTypeChanged();
+  return native_widget_ ? native_widget_->ShouldWindowContentsBeTransparent()
+                        : false;
 }
 
 void Widget::FrameTypeChanged() {
-  native_widget_->FrameTypeChanged();
+  if (native_widget_)
+    native_widget_->FrameTypeChanged();
 }
 
 const ui::Compositor* Widget::GetCompositor() const {
-  if (native_widget_destroyed_)
-    return nullptr;
-  return native_widget_->GetCompositor();
+  return native_widget_ ? native_widget_->GetCompositor() : nullptr;
 }
 
 const ui::Layer* Widget::GetLayer() const {
-  if (native_widget_destroyed_)
-    return nullptr;
-  return native_widget_->GetLayer();
+  return native_widget_ ? native_widget_->GetLayer() : nullptr;
 }
 
 void Widget::ReorderNativeViews() {
-  if (native_widget_destroyed_)
-    return;
-  native_widget_->ReorderNativeViews();
+  if (native_widget_)
+    native_widget_->ReorderNativeViews();
 }
 
 void Widget::LayerTreeChanged() {
@@ -1142,14 +1239,17 @@ void Widget::LayerTreeChanged() {
 }
 
 const NativeWidget* Widget::native_widget() const {
-  return native_widget_;
+  return native_widget_.get();
 }
 
 NativeWidget* Widget::native_widget() {
-  return native_widget_;
+  return native_widget_.get();
 }
 
 void Widget::SetCapture(View* view) {
+  if (!native_widget_)
+    return;
+
   if (!native_widget_->HasCapture()) {
     native_widget_->SetCapture();
 
@@ -1164,26 +1264,25 @@ void Widget::SetCapture(View* view) {
 }
 
 void Widget::ReleaseCapture() {
-  if (native_widget_->HasCapture())
+  if (native_widget_ && native_widget_->HasCapture())
     native_widget_->ReleaseCapture();
 }
 
 bool Widget::HasCapture() {
-  return native_widget_->HasCapture();
+  return native_widget_ ? native_widget_->HasCapture() : false;
 }
 
 TooltipManager* Widget::GetTooltipManager() {
-  if (native_widget_destroyed_)
-    return nullptr;
-  return native_widget_->GetTooltipManager();
+  return native_widget_ ? native_widget_->GetTooltipManager() : nullptr;
 }
 
 const TooltipManager* Widget::GetTooltipManager() const {
-  return native_widget_->GetTooltipManager();
+  return native_widget_ ? native_widget_->GetTooltipManager() : nullptr;
 }
 
 gfx::Rect Widget::GetWorkAreaBoundsInScreen() const {
-  return native_widget_->GetWorkAreaBoundsInScreen();
+  return native_widget_ ? native_widget_->GetWorkAreaBoundsInScreen()
+                        : gfx::Rect();
 }
 
 void Widget::SynthesizeMouseMoveEvent() {
@@ -1202,19 +1301,21 @@ void Widget::SynthesizeMouseMoveEvent() {
 }
 
 bool Widget::IsTranslucentWindowOpacitySupported() const {
-  return native_widget_->IsTranslucentWindowOpacitySupported();
+  return native_widget_ ? native_widget_->IsTranslucentWindowOpacitySupported()
+                        : false;
 }
 
 ui::GestureRecognizer* Widget::GetGestureRecognizer() {
-  return native_widget_->GetGestureRecognizer();
+  return native_widget_ ? native_widget_->GetGestureRecognizer() : nullptr;
 }
 
 ui::GestureConsumer* Widget::GetGestureConsumer() {
-  return native_widget_->GetGestureConsumer();
+  return native_widget_ ? native_widget_->GetGestureConsumer() : nullptr;
 }
 
 void Widget::OnSizeConstraintsChanged() {
-  native_widget_->OnSizeConstraintsChanged();
+  if (native_widget_)
+    native_widget_->OnSizeConstraintsChanged();
   if (non_client_view_)
     non_client_view_->SizeConstraintsChanged();
 }
@@ -1222,7 +1323,7 @@ void Widget::OnSizeConstraintsChanged() {
 void Widget::OnOwnerClosing() {}
 
 std::string Widget::GetName() const {
-  return native_widget_->GetName();
+  return native_widget_ ? native_widget_->GetName() : "";
 }
 
 base::CallbackListSubscription Widget::RegisterPaintAsActiveChangedCallback(
@@ -1234,7 +1335,7 @@ std::unique_ptr<Widget::PaintAsActiveLock> Widget::LockPaintAsActive() {
   const bool was_paint_as_active = ShouldPaintAsActive();
   ++paint_as_active_refcount_;
   if (ShouldPaintAsActive() != was_paint_as_active) {
-    paint_as_active_callbacks_.Notify();
+    NotifyPaintAsActiveChanged();
     if (parent() && !parent_paint_as_active_lock_)
       parent_paint_as_active_lock_ = parent()->LockPaintAsActive();
   }
@@ -1260,7 +1361,7 @@ void Widget::OnParentShouldPaintAsActiveChanged() {
   // |native_widget_| has already been deleted and |this| is being deleted so
   // that we don't have to handle the event and also it's unsafe to reference
   // |native_widget_| in this case.
-  if (native_widget_destroyed_)
+  if (!native_widget_)
     return;
 
   // |native_widget_active| is being updated in
@@ -1271,15 +1372,34 @@ void Widget::OnParentShouldPaintAsActiveChanged() {
   // this->ShouldPaintAsActive() changes iff the native widget is
   // inactive and there's no lock on this widget.
   if (!(native_widget_active_ || paint_as_active_refcount_))
-    paint_as_active_callbacks_.Notify();
+    NotifyPaintAsActiveChanged();
+}
+
+void Widget::NotifyPaintAsActiveChanged() {
+  paint_as_active_callbacks_.Notify();
+  if (native_widget_) {
+    native_widget_->PaintAsActiveChanged();
+  }
 }
 
 void Widget::SetNativeTheme(ui::NativeTheme* native_theme) {
+  // If `native_theme_` has been set for testing ensure the theme instance is
+  // not reset.
+  if (native_theme_set_for_testing_) {
+    return;
+  }
+
+  const bool is_update = native_theme_ && (native_theme_ != native_theme);
   native_theme_ = native_theme;
   native_theme_observation_.Reset();
   if (native_theme)
     native_theme_observation_.Observe(native_theme);
-  ThemeChanged();
+
+  if (is_update) {
+    OnNativeThemeUpdated(native_theme);
+  } else {
+    ThemeChanged();
+  }
 }
 
 int Widget::GetX() const {
@@ -1409,7 +1529,7 @@ bool Widget::OnNativeWidgetActivationChanged(bool active) {
   // Notify controls (e.g. LabelButton) and children widgets about the
   // paint-as-active change.
   if (ShouldPaintAsActive() != was_paint_as_active)
-    paint_as_active_callbacks_.Notify();
+    NotifyPaintAsActiveChanged();
 
   return true;
 }
@@ -1444,6 +1564,8 @@ void Widget::OnNativeWidgetCreated() {
   if (is_top_level())
     focus_manager_ = FocusManagerFactory::Create(this);
 
+  DCHECK(native_widget_);
+  DCHECK(widget_delegate_);
   native_widget_->InitModalType(widget_delegate_->GetModalType());
 
   for (WidgetObserver& observer : observers_)
@@ -1453,6 +1575,7 @@ void Widget::OnNativeWidgetCreated() {
 void Widget::OnNativeWidgetDestroying() {
   // Tell the focus manager (if any) that root_view is being removed
   // in case that the focused view is under this root view.
+  DCHECK(native_widget_);
   if (GetFocusManager() && root_view_)
     GetFocusManager()->ViewRemoved(root_view_.get());
   for (WidgetObserver& observer : observers_)
@@ -1465,15 +1588,25 @@ void Widget::OnNativeWidgetDestroying() {
 void Widget::OnNativeWidgetDestroyed() {
   for (WidgetObserver& observer : observers_)
     observer.OnWidgetDestroyed(this);
-  widget_delegate_->can_delete_this_ = true;
-  // `DeleteDelegate()` ends up destroying the object that `widget_delegate_`
-  // points to. Use `ExtractAsDangling()` to avoid having `widget_delegate_`
-  // briefly point to freed memory.
-  widget_delegate_.ExtractAsDangling()->DeleteDelegate();
-  // TODO(pbos): Replace this with native_widget_ = nullptr; and nullptr
-  // checking. This currently breaks on reentrant calls to CloseNow() that I'm
-  // too scared to fix right now.
-  native_widget_destroyed_ = true;
+  // TODO(kylixrd): Remove the references to owned_by_widget once widgets cease
+  // being able to "own" the delegate.
+  if (widget_delegate_) {
+    if (widget_delegate_->owned_by_widget()) {
+      widget_delegate_->DeleteDelegate();
+      widget_delegate_->can_delete_this_ = true;
+      owned_widget_delegate_.reset();
+    } else {
+      widget_delegate_->can_delete_this_ = true;
+      widget_delegate_->DeleteDelegate();
+    }
+  }
+  // Immediately reset the weak ptr. If NATIVE_WIDGET_OWNS_WIDGET destruction of
+  // the NativeWidget can destroy the Widget. We don't want to touch the
+  // NativeWidget during the destruction of the Widget either since some member
+  // variables on the NativeWidget may already be destroyed. In
+  // WIDGET_OWNS_NATIVE_WIDGET the NativeWidget will be cleaned up through
+  // |owned_native_widget_|
+  native_widget_.reset();
 }
 
 void Widget::OnNativeWidgetParentChanged(gfx::NativeView parent) {
@@ -1492,7 +1625,8 @@ gfx::Size Widget::GetMaximumSize() const {
 void Widget::OnNativeWidgetMove() {
   TRACE_EVENT0("ui", "Widget::OnNativeWidgetMove");
 
-  widget_delegate_->OnWidgetMove();
+  if (widget_delegate_)
+    widget_delegate_->OnWidgetMove();
   NotifyCaretBoundsChanged(GetInputMethod());
 
   for (WidgetObserver& observer : observers_)
@@ -1520,11 +1654,13 @@ void Widget::OnNativeWidgetWindowShowStateChanged() {
 }
 
 void Widget::OnNativeWidgetBeginUserBoundsChange() {
-  widget_delegate_->OnWindowBeginUserBoundsChange();
+  if (widget_delegate_)
+    widget_delegate_->OnWindowBeginUserBoundsChange();
 }
 
 void Widget::OnNativeWidgetEndUserBoundsChange() {
-  widget_delegate_->OnWindowEndUserBoundsChange();
+  if (widget_delegate_)
+    widget_delegate_->OnWindowEndUserBoundsChange();
 }
 
 void Widget::OnNativeWidgetAddedToCompositor() {}
@@ -1565,6 +1701,9 @@ void Widget::OnKeyEvent(ui::KeyEvent* event) {
 //                   RootView from anywhere in Widget. Use
 //                   SendEventToSink() instead. See crbug.com/348087.
 void Widget::OnMouseEvent(ui::MouseEvent* event) {
+  if (!native_widget_)
+    return;
+
   TRACE_EVENT0("ui", "Widget::OnMouseEvent");
 
   View* root_view = GetRootView();
@@ -1637,6 +1776,13 @@ void Widget::OnMouseEvent(ui::MouseEvent* event) {
       }
       return;
 
+    case ui::ET_MOUSE_ENTERED:
+      last_mouse_event_was_move_ = false;
+      if (root_view) {
+        root_view->OnMouseEntered(*event);
+      }
+      return;
+
     case ui::ET_MOUSE_EXITED:
       last_mouse_event_was_move_ = false;
       if (root_view)
@@ -1665,7 +1811,6 @@ void Widget::OnMouseCaptureLost() {
 }
 
 void Widget::OnScrollEvent(ui::ScrollEvent* event) {
-  // b/257997427 NOLINTNEXTLINE
   ui::ScrollEvent event_copy(*event);
   SendEventToSink(&event_copy);
 
@@ -1684,6 +1829,9 @@ void Widget::OnGestureEvent(ui::GestureEvent* event) {
 }
 
 bool Widget::ExecuteCommand(int command_id) {
+  if (!widget_delegate_)
+    return false;
+
   return widget_delegate_->ExecuteWindowsCommand(command_id);
 }
 
@@ -1695,6 +1843,9 @@ bool Widget::HasHitTestMask() const {
 }
 
 void Widget::GetHitTestMask(SkPath* mask) const {
+  if (!widget_delegate_)
+    return;
+
   DCHECK(mask);
   widget_delegate_->GetWidgetHitTestMask(mask);
 }
@@ -1709,7 +1860,7 @@ const Widget* Widget::AsWidget() const {
 
 bool Widget::SetInitialFocus(ui::WindowShowState show_state) {
   FocusManager* focus_manager = GetFocusManager();
-  if (!focus_manager)
+  if (!focus_manager || !widget_delegate_)
     return false;
   View* v = widget_delegate_->GetInitiallyFocusedView();
   if (!focus_on_creation_ || show_state == ui::SHOW_STATE_INACTIVE ||
@@ -1748,8 +1899,8 @@ bool Widget::ShouldDescendIntoChildForEventHandling(
 
   // Don't descend into |child| if there is a view with a Layer that contains
   // the point and is stacked above |child_layer|.
-  auto child_layer_iter = std::find(root_layer->children().begin(),
-                                    root_layer->children().end(), child_layer);
+  auto child_layer_iter =
+      base::ranges::find(root_layer->children(), child_layer);
   if (child_layer_iter == root_layer->children().end())
     return true;
 
@@ -1760,8 +1911,7 @@ bool Widget::ShouldDescendIntoChildForEventHandling(
     ui::Layer* layer = view->layer();
     DCHECK(layer);
     if (layer->visible() && layer->bounds().Contains(location)) {
-      auto root_layer_iter = std::find(root_layer->children().begin(),
-                                       root_layer->children().end(), layer);
+      auto root_layer_iter = base::ranges::find(root_layer->children(), layer);
       if (child_layer_iter > root_layer_iter) {
         // |child| is on top of the remaining layers, no need to continue.
         return true;
@@ -1806,15 +1956,13 @@ FocusSearch* Widget::GetFocusSearch() {
 FocusTraversable* Widget::GetFocusTraversableParent() {
   // We are a proxy to the root view, so we should be bypassed when traversing
   // up and as a result this should not be called.
-  NOTREACHED();
-  return nullptr;
+  NOTREACHED_NORETURN();
 }
 
 View* Widget::GetFocusTraversableParentView() {
   // We are a proxy to the root view, so we should be bypassed when traversing
   // up and as a result this should not be called.
-  NOTREACHED();
-  return nullptr;
+  NOTREACHED_NORETURN();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1825,15 +1973,33 @@ void Widget::OnNativeThemeUpdated(ui::NativeTheme* observed_theme) {
   ThemeChanged();
 }
 
+void Widget::SetColorModeOverride(
+    absl::optional<ui::ColorProviderKey::ColorMode> color_mode) {
+  color_mode_override_ = color_mode;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Widget, ui::ColorProviderSource:
 
-ui::ColorProviderManager::Key Widget::GetColorProviderKey() const {
-  ui::ColorProviderManager::Key key =
-      GetNativeTheme()->GetColorProviderKey(GetCustomTheme());
+ui::ColorProviderKey Widget::GetColorProviderKey() const {
+  // Generally all Widgets should inherit the key of their parent, falling back
+  // to the key set by the NativeTheme otherwise.
+  // TODO(crbug.com/1455535): `parent_` does not always resolve to the logical
+  // parent as expected here (e.g. bubbles). This should be addressed and the
+  // use of parent_ below replaced with something like GetLogicalParent().
+  ui::ColorProviderKey key =
+      parent_ ? parent_->GetColorProviderKey()
+              : GetNativeTheme()->GetColorProviderKey(GetCustomTheme());
+
+  // Widgets may have specific overrides set on the Widget itself that should
+  // apply specifically to themselves and their children, apply these here.
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   key.elevation_mode = background_elevation_;
 #endif
+  if (color_mode_override_.has_value()) {
+    key.color_mode = color_mode_override_.value();
+  }
+
   return key;
 }
 
@@ -1842,6 +2008,10 @@ const ui::ColorProvider* Widget::GetColorProvider() const {
       GetColorProviderKey());
 }
 
+ui::ColorProviderKey Widget::GetColorProviderKeyForTesting() const {
+  return GetColorProviderKey();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Widget, protected:
 
@@ -1887,7 +2057,8 @@ void Widget::SaveWindowPlacement() {
   // by go/crash) that in some circumstances we can end up here after
   // WM_DESTROY, at which point the window delegate is likely gone. So just
   // bail.
-  if (!widget_delegate_ || !widget_delegate_->ShouldSaveWindowPlacement())
+  if (!widget_delegate_ || !widget_delegate_->ShouldSaveWindowPlacement() ||
+      !native_widget_)
     return;
   ui::WindowShowState show_state = ui::SHOW_STATE_NORMAL;
   gfx::Rect bounds;
@@ -1940,7 +2111,7 @@ void Widget::SetInitialBoundsForFramelessWindow(const gfx::Rect& bounds) {
     // No initial bounds supplied, so size the window to its content and
     // center over its parent if preferred size is provided.
     gfx::Size size = contents_view->GetPreferredSize();
-    if (!size.IsEmpty())
+    if (!size.IsEmpty() && native_widget_)
       native_widget_->CenterWindow(size);
   } else {
     // Use the supplied initial bounds.
@@ -2018,7 +2189,7 @@ void Widget::UnlockPaintAsActive() {
     parent_paint_as_active_lock_.reset();
 
   if (ShouldPaintAsActive() != was_paint_as_active)
-    paint_as_active_callbacks_.Notify();
+    NotifyPaintAsActiveChanged();
 }
 
 void Widget::ClearFocusFromWidget() {
