@@ -5,7 +5,7 @@
 #ifndef WEBLAYER_PUBLIC_FULLSCREEN_DELEGATE_H_
 #define WEBLAYER_PUBLIC_FULLSCREEN_DELEGATE_H_
 
-#include "base/callback_forward.h"
+#include "base/functional/callback_forward.h"
 
 namespace weblayer {
 
